<?php

///////////////////////////////////////////////////////////////////////////

error_reporting(E_ALL);

date_default_timezone_set('UTC');

set_time_limit(0);

function hd_print($text)
{
#    echo($text . "\n");
}

function hd_error_silencer($severity, $msg, $filename, $linenum)
{ }

function hd_silence_warnings()
{
    error_reporting(0);
    set_error_handler("hd_error_silencer");
}

function hd_restore_warnings()
{
    restore_error_handler();
    error_reporting(E_STRICT | E_ALL);
}

function path_basename($str)
{
	return preg_replace('/^.*\//', '', $str);
}

function path_dirname($str)
{
	$str = rtrim($str, '/');
	$ndx = strrpos($str, '/');
	if ($ndx === false)
		return '';
	if ($ndx == 0)
		return '/';
	return substr($str, 0, $ndx);
}

function raw_json_encode($arr)
{
    $pattern = "/\\\\u([0-9a-fA-F]{4})/";
    $callback = function ($m) {
        return html_entity_decode("&#x$m[1];", ENT_QUOTES, 'UTF-8');
    };
    return str_replace('\\/', '/',
        preg_replace_callback($pattern, $callback,
            json_encode($arr)));
}

function make_pretty_json($json)
{
    $result = '';
    $level = 0;
    $in_quotes = false;
    $in_escape = false;
    $ends_line_level = NULL;
    $json_length = strlen( $json );

    for( $i = 0; $i < $json_length; $i++ ) {
        $char = $json[$i];
        $new_line_level = NULL;
        $post = "";
        if( $ends_line_level !== NULL ) {
            $new_line_level = $ends_line_level;
            $ends_line_level = NULL;
        }
        if ( $in_escape ) {
            $in_escape = false;
        } else if( $char === '"' ) {
            $in_quotes = !$in_quotes;
        } else if( ! $in_quotes ) {
            switch( $char ) {
                case '}': case ']':
                    $level--;
                    $ends_line_level = NULL;
                    $new_line_level = $level;
                    break;

                case '{': case '[':
                    $level++;
                case ',':
                    $ends_line_level = $level;
                    break;

                case ':':
                    $post = " ";
                    break;

                case " ": case "\t": case "\n": case "\r":
                    $char = "";
                    $ends_line_level = $new_line_level;
                    $new_line_level = NULL;
                    break;
            }
        } else if ( $char === '\\' ) {
            $in_escape = true;
        }
        if( $new_line_level !== NULL ) {
            $result .= "\n".str_repeat( "  ", $new_line_level );
        }
        $result .= $char.$post;
    }

    return $result;
}

///////////////////////////////////////////////////////////////////////////

require_once("lang.php");
require_once("country.php");
require_once("genre.php");
require_once("config_utils.php");
require_once("parse_utils.php");
require_once("path_helper.php");
require_once("system_config.php");
require_once("gui_common.php");
require_once("mdb_types.php");
require_once("dunemdb_api.php");
require_once("dunemdb_cache.php");
require_once("data_model.php");

///////////////////////////////////////////////////////////////////////////

Lang::static_init();
Country::static_init();
Genre::static_init();

///////////////////////////////////////////////////////////////////////////

interface MyDumper
{
    function set_response_headers();

    function get_table_header($titles, $page_size);
    function get_table_footer($titles, $page_size);
    function get_row($fields, $row_ndx);
}

///////////////////////////////////////////////////////////////////////////

class MyCsv implements MyDumper
{
    private $titles = null;

    public function set_response_headers()
    {
#        header('Content-Type: text/csv; charset=utf-8');
		header("Content-type: application/vnd.ms-excel; charset=utf-8");
		header("Content-disposition: attachment; filename=my_collection_dump.csv");
    }

    public function get_table_header($titles, $page_size)
    {
        $this->titles = $titles;

        $bom = chr(239) . chr(187) . chr(191);
        $res = $bom;
        foreach ($titles as $obj)
        {
            if (strlen($res) > 3)
                $res .= ',';
            $res .= $obj['caption'];
        }
        $res .= "\n";
        return $res;
    }

    public function get_table_footer($titles, $page_size)
    { return ""; }

    private function escape_value($v, $type)
    {
        if ($type != 'string')
            return $v;

        if (strpos($v, '"') === false &&
            strpos($v, ',') === false &&
            strpos($v, "\n") === false)
        {
            if (is_numeric($v))
                return '="' . $v . '"';
            return $v;
        }

        return '"' . str_replace('"', '""', $v) . '"';
    }

    public function get_row($fields, $row_ndx)
    {
        $res = '';
        for ($n = 0; $n < count($fields); $n++)
        {
            if ($res != '')
                $res .= ',';

            $v = $fields[$n];
            $type = $this->titles[$n]['type'];
            $res .= $this->escape_value($v, $type);
        }
        $res .= "\n";
        return $res;
    }
}

///////////////////////////////////////////////////////////////////////////

class MyHtml implements MyDumper
{
    private $mydir = null;
    private $titles = null;

    public function __construct()
    {
        $this->mydir = path_dirname($_SERVER["SCRIPT_FILENAME"]);
        if ($this->mydir === '')
            $this->mydir = '.';
    }

    public function set_response_headers()
    {
        header('Content-Type: text/html; charset=utf-8');
    }

    public function get_table_header($titles, $page_size)
    {
        $this->titles = $titles;
        $path = "$this->mydir/html_header";
        $doc = file_get_contents($path);

        $button_uri_prefix = $_SERVER['REQUEST_URI'];
        $button_uri_prefix = preg_replace(
            '|button=[a-z0-9]*&|', '', $button_uri_prefix);
        $button_uri_prefix = preg_replace(
            '|[?&]button=[a-z0-9]*|', '', $button_uri_prefix);
        $button_uri_prefix .=
            (false === strpos($button_uri_prefix, '?') ?
             '?button=' : '&button=');

        return str_replace('@@@DEBUG@@@', '',
            str_replace('@@@BUTTON_URI_PREFIX@@@', $button_uri_prefix, $doc));
    }

    public function get_table_footer($titles, $page_size)
    {
        $this->titles = $titles;
        $path = "$this->mydir/html_footer";
        $doc = file_get_contents($path);

        $cols = array();
        for ($n = 0; $n < count($titles); $n++)
        {
            $obj = $titles[$n];
            $cols[] = array(
                'title' => $obj['caption'],
                'field' => "f" . ($n + 1),
                'sorter' => $obj['type'],
            );
        }

        $cols_str = make_pretty_json(json_encode($cols));

        return str_replace('@@@COLUMNS@@@', $cols_str,
            str_replace('@@@PAGE_SIZE@@@', $page_size, $doc));
    }

    public function get_row($fields, $row_ndx)
    {
        $row = array('id' => $row_ndx + 1);
        for ($n = 0; $n < count($fields); $n++)
        {
            $v = $fields[$n];
            $type = $this->titles[$n]['type'];
            $key = "f" . ($n + 1);
            $row[$key] = $type == 'number' ?
                (is_numeric($v) ? intval($v) : strval($v)) :
                strval($v);
        }
//        return make_pretty_json(raw_json_encode($row)) . ",\n";
        return raw_json_encode($row) . ",\n";
    }
}

///////////////////////////////////////////////////////////////////////////

$cmp_order = array(2, 4);

function cmp_rows($a, $b)
{
    global $cmp_order;
    foreach ($cmp_order as $ndx)
    {
        $sign = $ndx < 0 ? -1 : 1;
        $ndx = abs($ndx) - 1;
        $n = strnatcmp($a[$ndx], $b[$ndx]);
        if ($n != 0)
            return $n * $sign;
    }
    return 0;
}

function get_content_id($movie, $file)
{
    if (!isset($movie))
        return '';

    $s = isset($file->s) && $file->s >= 0 ? $file->s : '';
    $e = isset($file->e) && $file->e >= 0 ? $file->e : '';
    return $s === '' && $e === '' ?
        $movie->id : "$movie->id:$s:$e";
}

function is_suitable_by_cond($v, $cond)
{
    $op = $cond[1];

    if ($op == 'empty')
        return strval($v) == ''; 
    if ($op == 'nonempty')
        return strval($v) != ''; 

    if (count($cond) != 3)
        return false;

    if ($op == 'eq')
        return strval($v) == $cond[2];
    if ($op == 'substr')
        return false !== strpos(strval($v), $cond[2]);
    if ($op == 'ne')
        return strval($v) != $cond[2];
    if ($op == 'ge')
        return intval($v) >= $cond[2];
    if ($op == 'le')
        return intval($v) <= $cond[2];

    return false;
}

function is_suitable_by_filter($row, $filter)
{
    foreach ($filter as $cond)
    {
        if ($cond[1] == 'false')
            return false;

        $v = $row[$cond[0] - 1];
        if (!is_suitable_by_cond($v, $cond))
            return false;
    }
    return true;
}

function create_filter_from_string($filter_str, $num_cols)
{
    $filter = array();
    if ($filter_str == '')
        return $filter;

    $arr = explode(',', $filter_str);
    foreach ($arr as $f)
    {
        $cond = explode(':', $f);

        if (count($cond) < 2)
        { $filter[] = array(1, 'false'); break; }

        $field_id = $cond[0];
        if (!is_numeric($field_id))
        { $filter[] = array(1, 'false'); break; }
        if ($field_id < 1 || $field_id > $num_cols)
        { $filter[] = array(1, 'false'); break; }

        $filter[] = $cond;
    }

    return $filter;
}

function create_order_from_string($order_str, $num_cols)
{
    $order = array();
    if ($order_str == 'none')
        return $order;

    $order_arr = explode(',', $order_str);
    foreach ($order_arr as $v)
    {
        if (!is_numeric($v) || $v == 0 || abs($v) > $num_cols)
            break;
        $order[] = intval($v);
    }
    if (count($order) == 0)
        $order = array(2, 4);

    return $order;
}

function calculate_group_cols(&$rows)
{
    $ndups_by_cid = array();
    $dup_n_by_cid = array();
    foreach ($rows as $row)
    {
        $content_id = $row[11];
        $ndups = isset($ndups_by_cid[$content_id]) ?
            $ndups_by_cid[$content_id] : 0;
        $ndups_by_cid[$content_id] = $ndups + 1;
        $dup_n_by_cid[$content_id] = 0;
    }

    foreach ($rows as &$row)
    {
        $content_id = $row[11];
        $ndups = $ndups_by_cid[$content_id];
        $dup_n = $dup_n_by_cid[$content_id] + 1;
        $dup_n_by_cid[$content_id] = $dup_n;
        $row[12] = $ndups;
        $row[13] = $dup_n;
    }
}

function apply_filter($rows, $filter)
{
    if (count($filter) == 0)
        return $rows;

    $filtered_rows = array();
    $row_id = 1;
    foreach ($rows as $row)
    {
        if (!is_suitable_by_filter($row, $filter))
            continue;

        $filtered_rows[] = $row;
        $row[0] = $row_id;
        $row_id++;
    }

    return $filtered_rows;
}

function dump(MyDumper $dumper, $from, $limit, $order_str,
    $pre_filter_str, $filter_str, $page_size)
{
    global $cmp_order;

    $titles = array(
        array('caption' => 'N', 'type' => 'number'),
        array('caption' => 'MyCollection Folder', 'type' => 'string'),
        array('caption' => 'File Hash', 'type' => 'string'),
        array('caption' => 'File Path', 'type' => 'string'),
        array('caption' => 'Movie Title', 'type' => 'string'),
        array('caption' => 'Movie Year', 'type' => 'number'),
        array('caption' => 'Season N', 'type' => 'number'),
        array('caption' => 'Episode N', 'type' => 'number'),
        array('caption' => 'Movie ID', 'type' => 'string'),
        array('caption' => 'Match Confirmed', 'type' => 'string'),
        array('caption' => 'Association Type', 'type' => 'string'),
        array('caption' => 'Content ID', 'type' => 'string'),
        array('caption' => 'Count of Duplicates', 'type' => 'number'),
        array('caption' => 'N of Duplicate', 'type' => 'number'),
    );

    $from = max(0, $from);
    $max_row = $from + $limit;

    $dumper->set_response_headers();

    echo($dumper->get_table_header($titles, $page_size));

    $cfg = SystemConfig::get_instance();
    $cfg->reload();

    $model = DataModel::get_instance();
    $model->ensure_loaded();

    $cache = DunemdbCache::get_instance();

    $folders = $model->get_folders();
    $folder_files = array();
    foreach ($folders as $folder)
    {
        $folder_files[$folder->id] = $model->get_all_folder_files(
            DataModel::FILTER_ALL_FILES, $folder->id);
    }

    $rows = array();
    $row_id = 1;
    foreach ($folders as $folder)
    {
        $files = $folder_files[$folder->id];
        foreach ($files as $file)
        {
            $movie = isset($file->movie_id) ?
                $cache->get_and_cache_short_movie($file->movie_id) : null;
            $title = isset($movie) ? $movie->title : '';
            $year = isset($movie) && isset($movie->year) && $movie->year > 0 ?
                $movie->year : '';
            $ext_id = isset($movie) ? $movie->get_ext_id() : '';

            $s = isset($file->s) && $file->s >= 0 ? $file->s : '';
            $e = isset($file->e) && $file->e >= 0 ? $file->e : '';
            $ass_type = "unknown";
            if (isset($file->ass_type))
                $ass_type = DunemdbApi::ass_type_to_content($file->ass_type);
            $mc = isset($file->match_confirmed) && $file->match_confirmed ?
                "yes" : "no";

            $content_id = get_content_id($movie, $file);

            $rows[] = array(
                $row_id,
                $folder->caption,
                $file->hash,
                $file->path,
                $title,
                $year,
                $s,
                $e,
                $ext_id,
                $mc,
                $ass_type,
                $content_id,
                0, // Group column
                0, // Group column
            );
            $row_id++;
        }
    }

    $pre_filter = create_filter_from_string(
        $pre_filter_str, count($titles));
    $rows = apply_filter($rows, $pre_filter);

    calculate_group_cols($rows);

    $filter = create_filter_from_string(
        $filter_str, count($titles));
    $rows = apply_filter($rows, $filter);

    $cmp_order = create_order_from_string(
        $order_str, count($titles));
    if ($order_str != 'none')
        usort($rows, 'cmp_rows');

    if ($limit == 0)
        $max_row = count($rows);

    for ($row_ndx = $from; $row_ndx < $max_row; $row_ndx++)
    {
        $row = $rows[$row_ndx];
        $row[0] = $row_ndx + 1;
        echo($dumper->get_row($row, $row_ndx));
    }

    echo($dumper->get_table_footer($titles, $page_size));
}

$format = isset($_REQUEST['format']) ? $_REQUEST['format'] : 'html';
$limit = isset($_REQUEST['limit']) ? $_REQUEST['limit'] : 0;
$limit = intval($limit);
$from = isset($_REQUEST['from']) ? $_REQUEST['from'] : 0;
$from = intval($from);
$order = isset($_REQUEST['order']) ? $_REQUEST['order'] : '2,4';
$pre_filter = isset($_REQUEST['pre_filter']) ? $_REQUEST['pre_filter'] : '';
$filter = isset($_REQUEST['filter']) ? $_REQUEST['filter'] : '';
$page_size = isset($_REQUEST['page_size']) ? $_REQUEST['page_size'] : 32;
$page_size = max(10, intval($page_size));

$button = isset($_REQUEST['button']) ? $_REQUEST['button'] : null;
if ($button == 'filter1')
{
    $order = '12,14';
    $pre_filter = '11:eq:movie';
    $filter = '13:ge:2';
}

if ($format == 'csv')
    $dumper = new MyCsv();
else if ($format == 'html')
    $dumper = new MyHtml();
else
{
    echo("Invalid format: $format\n");
    echo("Valid formats: csv, html\n");
    exit(0);
}

dump($dumper, $from, $limit, $order, $pre_filter, $filter, $page_size);

///////////////////////////////////////////////////////////////////////////

?>
