<?php

class ParseUtils
{
    public static function decode_html_entities($text)
    {
        return html_entity_decode($text, ENT_QUOTES, 'UTF-8');
    }

    public static function decode_html($html)
    {
        $html = preg_replace('/<li[^>]*>/', '•  ', $html);
        $html = preg_replace('@</li>@', "\n", $html);
        $html = preg_replace('@<br/*>@', "\n", $html);
        return self::decode_html_entities(strip_tags($html));
    }

    private static function opt_decode_string($string_node, $decode_mode)
    {
        switch ($decode_mode)
        {
            case 1:
                return self::decode_html_entities(strval($string_node));
            case 2:
                return self::decode_html(strval($string_node));
            default:
                return strval($string_node);
        }
    }

    private static function str_has_russian_letters($str)
    {
        if (preg_match('/[абвгдеёжзийклмнопрстуфхцчшщъыьэюя]/ui', $str))
            return true;

        return false;
    }

    public static function parse_multilang_string(&$json, $param_name,
        $lang, $error_or_missing_value, $decode_mode, $use_ru_if_no_en,
        &$out_effective_lang=null)
    {
        if ($out_effective_lang !== null)
            $out_effective_lang = null;

        if (!isset($json->{$param_name}))
            return $error_or_missing_value;

        $val = &$json->{$param_name};
        if (is_scalar($val))
            return self::opt_decode_string($val, $decode_mode);

        if (!is_object($val))
            return $error_or_missing_value;

        if (isset($val->{$lang}) && strval($val->{$lang}) !== '')
        {
            $out_effective_lang = $lang;
            return self::opt_decode_string($val->{$lang}, $decode_mode);
        }

        if ($lang == 'uk' && isset($val->ru) && strval($val->ru) !== '')
        {
            $out_effective_lang = 'ru';
            return self::opt_decode_string($val->ru, $decode_mode);
        }

        if ($lang != 'en' && isset($val->en) && strval($val->en) !== '')
        {
            $out_effective_lang = 'en';
            return self::opt_decode_string($val->en, $decode_mode);
        }

        foreach ($val as $k => $v)
        {
            if (strval($v) === '')
                continue;

            $value = self::opt_decode_string($v, $decode_mode);
            if ($k == 'ru' && !$use_ru_if_no_en && self::str_has_russian_letters($value))
                continue;

            $out_effective_lang = $k;
            return $value;
        }

        return $error_or_missing_value;
    }
}

?>
