<?php

class SystemConfig
{
    private static $instance = null;

    public static function get_instance()
    {
        if (!isset(self::$instance))
            self::$instance = new SystemConfig();
        return self::$instance;
    }

    private function __construct()
    { }

    ///////////////////////////////////////////////////////////////////////

    public $lang_id;
    public $lang_code;
    public $widget_country;
    public $country;
    public $country_code_uc;
    public $location_info_loaded;

    private function reload_settings()
    {
        $cfg = ConfigUtils::load_config();
        $this->lang_id = $cfg->lang_id;
        $this->lang_code = $cfg->lang_code;
        $this->widget_country = $cfg->widget_country;

        hd_print("Interface language: $this->lang_id ($this->lang_code)");
        hd_print("Widget country: $this->widget_country");
    }

    private function reload_location_info()
    {
        $this->country = "";
        $this->country_code_uc = "";

        $this->ensure_location_info_fetched();

        if (is_file("/tmp/location_info.properties"))
            $location_info_path = "/tmp/location_info.properties";
        else if (is_file("/config/location_info.properties"))
            $location_info_path = "/config/location_info.properties";
        else
            return;

        $lines = file($location_info_path,
            FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
        foreach ($lines as $line)
        {
            if (preg_match('/^country =\s*(\S.*\S)\s*$/', $line, $matches))
            {
                $this->country = $matches[1];
                if (preg_match('/(\S[^,]*)\s*,/', $this->country, $matches))
                    $this->country = $matches[1];
                hd_print("Country: " . $this->country);
            }

            if (preg_match('/^country_code_2 =\s*(\S\S)\s*$/', $line, $matches))
            {
                $this->country_code_uc = strtoupper($matches[1]);
                hd_print("Country code: " . $this->country_code_uc);
            }
        }

        $this->location_info_loaded = 1;
    }

    public function reload()
    {
        $this->reload_settings();
        $this->reload_location_info();

        // TODO: remove stub
        if ($this->country == '' && $this->country_code_uc == '')
        {
            $this->country_code_uc = 'RU';
            $this->country = 'Russian Federation';
            hd_print("Using dummy country code: " . $this->country_code_uc);
        }
    }

    public function ensure_loaded()
    {
        if (isset($this->lang_id))
            return;

        $this->reload();
    }

    public function ensure_location_info_loaded()
    {
        if (isset($this->location_info_loaded))
            return;
        $this->reload_location_info();
    }

    public function ensure_location_info_fetched()
    {
        if ($this->is_location_info_fetched())
            return;
        hd_print("Location info is not fetched, spawning /firmware/scripts/run_get_location_info.sh");
        system("/firmware/scripts/run_get_location_info.sh &");
        hd_print("Waiting for location info...");
        for ($i = 0; $i < 10; $i++)
        {
            usleep(200000);
            if ($this->is_location_info_fetched())
            {
                hd_print("Location info fetched OK");
                return;
            }
        }
        hd_print("Location info is still not fetched");
    }

    public function is_location_info_fetched()
    {
        if (is_file("/tmp/location_info.properties"))
            return true;
        if (is_file("/config/location_info.properties"))
            return true;
        return false;
    }
}

?>
